"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from gusto_app_integration import models, utils
from gusto_app_integration._hooks import HookContext
from gusto_app_integration.types import OptionalNullable, UNSET
from typing import Any, List, Mapping, Optional, Union


class Payrolls(BaseSDK):
    def get_for_company(
        self,
        *,
        company_id: str,
        processing_statuses: Optional[List[models.ProcessingStatuses]] = None,
        payroll_types: Optional[List[models.PayrollTypes]] = None,
        include: Optional[
            List[models.GetV1CompaniesCompanyIDPayrollsQueryParamInclude]
        ] = None,
        start_date: Optional[str] = None,
        end_date: Optional[str] = None,
        sort_order: Optional[models.SortOrder] = None,
        page: Optional[int] = None,
        per: Optional[int] = None,
        x_gusto_api_version: Optional[
            models.VersionHeader
        ] = models.VersionHeader.TWO_THOUSAND_AND_TWENTY_FOUR_MINUS_04_MINUS_01,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> List[models.PayrollMinimal]:
        r"""Get all payrolls for a company

        Returns a list of payrolls for a company. You can change the payrolls returned by updating the processing_status, payroll_types, start_date, & end_date params.

        By default, will return processed, regular payrolls for the past 6 months.

        Notes:
        * Dollar amounts are returned as string representations of numeric decimals, are represented to the cent.
        * end_date can be at most 3 months in the future and start_date and end_date can't be more than 1 year apart.

        scope: `payrolls:read`

        :param company_id: The UUID of the company
        :param processing_statuses: Whether to include processed and/or unprocessed payrolls in the response, defaults to processed, for multiple attributes comma separate the values, i.e. `?processing_statuses=processed,unprocessed`
        :param payroll_types: Whether to include regular and/or off_cycle payrolls in the response, defaults to regular, for multiple attributes comma separate the values, i.e. `?payroll_types=regular,off_cycle`
        :param include: Include the requested attribute in the response. The risk_blockers option will include submission_blockers and credit_blockers if applicable. In v2023-04-01 totals are no longer included by default. For multiple attributes comma separate the values, i.e. `?include=totals,payroll_status_meta`
        :param start_date: Return payrolls whose pay period is after the start date
        :param end_date: Return payrolls whose pay period is before the end date. If left empty, defaults to today's date.
        :param sort_order: A string indicating whether to sort resulting events in ascending (asc) or descending (desc) chronological order. Events are sorted by their `timestamp`. Defaults to asc if left empty.
        :param page: The page that is requested. When unspecified, will load all objects unless endpoint forces pagination.
        :param per: Number of objects per page. For majority of endpoints will default to 25
        :param x_gusto_api_version: Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetV1CompaniesCompanyIDPayrollsRequest(
            company_id=company_id,
            processing_statuses=processing_statuses,
            payroll_types=payroll_types,
            include=include,
            start_date=start_date,
            end_date=end_date,
            sort_order=sort_order,
            page=page,
            per=per,
            x_gusto_api_version=x_gusto_api_version,
        )

        req = self._build_request(
            method="GET",
            path="/v1/companies/{company_id}/payrolls",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="get-v1-companies-company_id-payrolls",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["404", "4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, List[models.PayrollMinimal])
        if utils.match_response(http_res, ["404", "4XX"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def get_for_company_async(
        self,
        *,
        company_id: str,
        processing_statuses: Optional[List[models.ProcessingStatuses]] = None,
        payroll_types: Optional[List[models.PayrollTypes]] = None,
        include: Optional[
            List[models.GetV1CompaniesCompanyIDPayrollsQueryParamInclude]
        ] = None,
        start_date: Optional[str] = None,
        end_date: Optional[str] = None,
        sort_order: Optional[models.SortOrder] = None,
        page: Optional[int] = None,
        per: Optional[int] = None,
        x_gusto_api_version: Optional[
            models.VersionHeader
        ] = models.VersionHeader.TWO_THOUSAND_AND_TWENTY_FOUR_MINUS_04_MINUS_01,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> List[models.PayrollMinimal]:
        r"""Get all payrolls for a company

        Returns a list of payrolls for a company. You can change the payrolls returned by updating the processing_status, payroll_types, start_date, & end_date params.

        By default, will return processed, regular payrolls for the past 6 months.

        Notes:
        * Dollar amounts are returned as string representations of numeric decimals, are represented to the cent.
        * end_date can be at most 3 months in the future and start_date and end_date can't be more than 1 year apart.

        scope: `payrolls:read`

        :param company_id: The UUID of the company
        :param processing_statuses: Whether to include processed and/or unprocessed payrolls in the response, defaults to processed, for multiple attributes comma separate the values, i.e. `?processing_statuses=processed,unprocessed`
        :param payroll_types: Whether to include regular and/or off_cycle payrolls in the response, defaults to regular, for multiple attributes comma separate the values, i.e. `?payroll_types=regular,off_cycle`
        :param include: Include the requested attribute in the response. The risk_blockers option will include submission_blockers and credit_blockers if applicable. In v2023-04-01 totals are no longer included by default. For multiple attributes comma separate the values, i.e. `?include=totals,payroll_status_meta`
        :param start_date: Return payrolls whose pay period is after the start date
        :param end_date: Return payrolls whose pay period is before the end date. If left empty, defaults to today's date.
        :param sort_order: A string indicating whether to sort resulting events in ascending (asc) or descending (desc) chronological order. Events are sorted by their `timestamp`. Defaults to asc if left empty.
        :param page: The page that is requested. When unspecified, will load all objects unless endpoint forces pagination.
        :param per: Number of objects per page. For majority of endpoints will default to 25
        :param x_gusto_api_version: Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetV1CompaniesCompanyIDPayrollsRequest(
            company_id=company_id,
            processing_statuses=processing_statuses,
            payroll_types=payroll_types,
            include=include,
            start_date=start_date,
            end_date=end_date,
            sort_order=sort_order,
            page=page,
            per=per,
            x_gusto_api_version=x_gusto_api_version,
        )

        req = self._build_request_async(
            method="GET",
            path="/v1/companies/{company_id}/payrolls",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="get-v1-companies-company_id-payrolls",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["404", "4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, List[models.PayrollMinimal])
        if utils.match_response(http_res, ["404", "4XX"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    def get(
        self,
        *,
        company_id: str,
        payroll_id: str,
        include: Optional[
            List[models.GetV1CompaniesCompanyIDPayrollsPayrollIDQueryParamInclude]
        ] = None,
        x_gusto_api_version: Optional[
            models.VersionHeader
        ] = models.VersionHeader.TWO_THOUSAND_AND_TWENTY_FOUR_MINUS_04_MINUS_01,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.Payroll:
        r"""Get a single payroll

        Returns a payroll. If payroll is calculated or processed, will return employee_compensations and totals.

        Notes:
        * Hour and dollar amounts are returned as string representations of numeric decimals.
        * Hours are represented to the thousands place; dollar amounts are represented to the cent.
        * Every eligible compensation is returned for each employee. If no data has yet be inserted for a given field, it defaults to “0.00” (for fixed amounts) or “0.000” (for hours ).
        * To return future payrolls, you must include an `end_date` in the future.
        * When include parameter with benefits value is passed, employee_benefits:read scope is required to return benefits
        * Benefits containing PHI are only visible with the `employee_benefits:read:phi` scope

        scope: `payrolls:read`

        :param company_id: The UUID of the company
        :param payroll_id: The UUID of the payroll
        :param include: Include the requested attribute in the response, for multiple attributes comma separate the values, i.e. `?include=benefits,deductions,taxes`
        :param x_gusto_api_version: Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetV1CompaniesCompanyIDPayrollsPayrollIDRequest(
            company_id=company_id,
            payroll_id=payroll_id,
            include=include,
            x_gusto_api_version=x_gusto_api_version,
        )

        req = self._build_request(
            method="GET",
            path="/v1/companies/{company_id}/payrolls/{payroll_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="get-v1-companies-company_id-payrolls-payroll_id",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["404", "4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, models.Payroll)
        if utils.match_response(http_res, ["404", "4XX"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def get_async(
        self,
        *,
        company_id: str,
        payroll_id: str,
        include: Optional[
            List[models.GetV1CompaniesCompanyIDPayrollsPayrollIDQueryParamInclude]
        ] = None,
        x_gusto_api_version: Optional[
            models.VersionHeader
        ] = models.VersionHeader.TWO_THOUSAND_AND_TWENTY_FOUR_MINUS_04_MINUS_01,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.Payroll:
        r"""Get a single payroll

        Returns a payroll. If payroll is calculated or processed, will return employee_compensations and totals.

        Notes:
        * Hour and dollar amounts are returned as string representations of numeric decimals.
        * Hours are represented to the thousands place; dollar amounts are represented to the cent.
        * Every eligible compensation is returned for each employee. If no data has yet be inserted for a given field, it defaults to “0.00” (for fixed amounts) or “0.000” (for hours ).
        * To return future payrolls, you must include an `end_date` in the future.
        * When include parameter with benefits value is passed, employee_benefits:read scope is required to return benefits
        * Benefits containing PHI are only visible with the `employee_benefits:read:phi` scope

        scope: `payrolls:read`

        :param company_id: The UUID of the company
        :param payroll_id: The UUID of the payroll
        :param include: Include the requested attribute in the response, for multiple attributes comma separate the values, i.e. `?include=benefits,deductions,taxes`
        :param x_gusto_api_version: Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetV1CompaniesCompanyIDPayrollsPayrollIDRequest(
            company_id=company_id,
            payroll_id=payroll_id,
            include=include,
            x_gusto_api_version=x_gusto_api_version,
        )

        req = self._build_request_async(
            method="GET",
            path="/v1/companies/{company_id}/payrolls/{payroll_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="get-v1-companies-company_id-payrolls-payroll_id",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["404", "4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, models.Payroll)
        if utils.match_response(http_res, ["404", "4XX"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    def update(
        self,
        *,
        company_id: str,
        payroll_id: str,
        employee_compensations: Union[
            List[models.EmployeeCompensations],
            List[models.EmployeeCompensationsTypedDict],
        ],
        x_gusto_api_version: Optional[
            models.VersionHeader
        ] = models.VersionHeader.TWO_THOUSAND_AND_TWENTY_FOUR_MINUS_04_MINUS_01,
        withholding_pay_period: Optional[models.WithholdingPayPeriod] = None,
        skip_regular_deductions: Optional[bool] = None,
        fixed_withholding_rate: Optional[bool] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.PayrollPrepared:
        r"""Update a payroll by ID

        This endpoint allows you to update information for one or more employees for a specific **unprocessed** payroll.  You can think of the **unprocessed**
        payroll object as a template of fields that you can update.  You cannot modify the structure of the payroll object through this endpoint, only values
        of the fields included in the payroll.  If you do not include specific employee compensations or fixed/hourly compensations in your request body, they
        will not be removed from the payroll.

        scope: `payrolls:write`

        :param company_id: The UUID of the company
        :param payroll_id: The UUID of the payroll
        :param employee_compensations:
        :param x_gusto_api_version: Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used.
        :param withholding_pay_period: The payment schedule tax rate the payroll is based on. Only relevant for off-cycle payrolls.
        :param skip_regular_deductions: Block regular deductions and contributions for this payroll. Only relevant for off-cycle payrolls.
        :param fixed_withholding_rate: Enable taxes to be withheld at the IRS's required rate of 22% for federal income taxes. State income taxes will be taxed at the state's supplemental tax rate. Otherwise, we'll sum the entirety of the employee's wages and withhold taxes on the entire amount at the rate for regular wages. Only relevant for off-cycle payrolls.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.PutV1CompaniesCompanyIDPayrollsRequest(
            company_id=company_id,
            payroll_id=payroll_id,
            x_gusto_api_version=x_gusto_api_version,
            request_body=models.PutV1CompaniesCompanyIDPayrollsRequestBody(
                employee_compensations=utils.get_pydantic_model(
                    employee_compensations, List[models.EmployeeCompensations]
                ),
                withholding_pay_period=withholding_pay_period,
                skip_regular_deductions=skip_regular_deductions,
                fixed_withholding_rate=fixed_withholding_rate,
            ),
        )

        req = self._build_request(
            method="PUT",
            path="/v1/companies/{company_id}/payrolls/{payroll_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.request_body,
                False,
                False,
                "json",
                models.PutV1CompaniesCompanyIDPayrollsRequestBody,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="put-v1-companies-company_id-payrolls",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, models.PayrollPrepared)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.UnprocessableEntityErrorObjectData
            )
            raise models.UnprocessableEntityErrorObject(data=response_data)
        if utils.match_response(http_res, ["404", "4XX"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def update_async(
        self,
        *,
        company_id: str,
        payroll_id: str,
        employee_compensations: Union[
            List[models.EmployeeCompensations],
            List[models.EmployeeCompensationsTypedDict],
        ],
        x_gusto_api_version: Optional[
            models.VersionHeader
        ] = models.VersionHeader.TWO_THOUSAND_AND_TWENTY_FOUR_MINUS_04_MINUS_01,
        withholding_pay_period: Optional[models.WithholdingPayPeriod] = None,
        skip_regular_deductions: Optional[bool] = None,
        fixed_withholding_rate: Optional[bool] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.PayrollPrepared:
        r"""Update a payroll by ID

        This endpoint allows you to update information for one or more employees for a specific **unprocessed** payroll.  You can think of the **unprocessed**
        payroll object as a template of fields that you can update.  You cannot modify the structure of the payroll object through this endpoint, only values
        of the fields included in the payroll.  If you do not include specific employee compensations or fixed/hourly compensations in your request body, they
        will not be removed from the payroll.

        scope: `payrolls:write`

        :param company_id: The UUID of the company
        :param payroll_id: The UUID of the payroll
        :param employee_compensations:
        :param x_gusto_api_version: Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used.
        :param withholding_pay_period: The payment schedule tax rate the payroll is based on. Only relevant for off-cycle payrolls.
        :param skip_regular_deductions: Block regular deductions and contributions for this payroll. Only relevant for off-cycle payrolls.
        :param fixed_withholding_rate: Enable taxes to be withheld at the IRS's required rate of 22% for federal income taxes. State income taxes will be taxed at the state's supplemental tax rate. Otherwise, we'll sum the entirety of the employee's wages and withhold taxes on the entire amount at the rate for regular wages. Only relevant for off-cycle payrolls.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.PutV1CompaniesCompanyIDPayrollsRequest(
            company_id=company_id,
            payroll_id=payroll_id,
            x_gusto_api_version=x_gusto_api_version,
            request_body=models.PutV1CompaniesCompanyIDPayrollsRequestBody(
                employee_compensations=utils.get_pydantic_model(
                    employee_compensations, List[models.EmployeeCompensations]
                ),
                withholding_pay_period=withholding_pay_period,
                skip_regular_deductions=skip_regular_deductions,
                fixed_withholding_rate=fixed_withholding_rate,
            ),
        )

        req = self._build_request_async(
            method="PUT",
            path="/v1/companies/{company_id}/payrolls/{payroll_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.request_body,
                False,
                False,
                "json",
                models.PutV1CompaniesCompanyIDPayrollsRequestBody,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="put-v1-companies-company_id-payrolls",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, models.PayrollPrepared)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.UnprocessableEntityErrorObjectData
            )
            raise models.UnprocessableEntityErrorObject(data=response_data)
        if utils.match_response(http_res, ["404", "4XX"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    def prepare(
        self,
        *,
        company_id: str,
        payroll_id: str,
        x_gusto_api_version: Optional[
            models.VersionHeader
        ] = models.VersionHeader.TWO_THOUSAND_AND_TWENTY_FOUR_MINUS_04_MINUS_01,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.PayrollPrepared:
        r"""Prepare a payroll for update

        This endpoint will build the payroll and get it ready for making updates. This includes adding/removing eligible employees from the Payroll and updating the check_date, payroll_deadline, and payroll_status_meta dates & times.

        Notes:
        * Will null out calculated_at & totals if a payroll has already been calculated.
        * Will return the version param used for updating the payroll

        scope: `payrolls:write`

        :param company_id: The UUID of the company
        :param payroll_id: The UUID of the payroll
        :param x_gusto_api_version: Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.PutV1CompaniesCompanyIDPayrollsPayrollIDPrepareRequest(
            company_id=company_id,
            payroll_id=payroll_id,
            x_gusto_api_version=x_gusto_api_version,
        )

        req = self._build_request(
            method="PUT",
            path="/v1/companies/{company_id}/payrolls/{payroll_id}/prepare",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="put-v1-companies-company_id-payrolls-payroll_id-prepare",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["404", "4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, models.PayrollPrepared)
        if utils.match_response(http_res, ["404", "4XX"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def prepare_async(
        self,
        *,
        company_id: str,
        payroll_id: str,
        x_gusto_api_version: Optional[
            models.VersionHeader
        ] = models.VersionHeader.TWO_THOUSAND_AND_TWENTY_FOUR_MINUS_04_MINUS_01,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.PayrollPrepared:
        r"""Prepare a payroll for update

        This endpoint will build the payroll and get it ready for making updates. This includes adding/removing eligible employees from the Payroll and updating the check_date, payroll_deadline, and payroll_status_meta dates & times.

        Notes:
        * Will null out calculated_at & totals if a payroll has already been calculated.
        * Will return the version param used for updating the payroll

        scope: `payrolls:write`

        :param company_id: The UUID of the company
        :param payroll_id: The UUID of the payroll
        :param x_gusto_api_version: Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.PutV1CompaniesCompanyIDPayrollsPayrollIDPrepareRequest(
            company_id=company_id,
            payroll_id=payroll_id,
            x_gusto_api_version=x_gusto_api_version,
        )

        req = self._build_request_async(
            method="PUT",
            path="/v1/companies/{company_id}/payrolls/{payroll_id}/prepare",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="put-v1-companies-company_id-payrolls-payroll_id-prepare",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["404", "4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, models.PayrollPrepared)
        if utils.match_response(http_res, ["404", "4XX"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .versionheader import VersionHeader
from gusto_app_integration.types import BaseModel
from gusto_app_integration.utils import (
    FieldMetadata,
    HeaderMetadata,
    PathParamMetadata,
    RequestMetadata,
)
import pydantic
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class PutAddPeopleToDepartmentEmployeesTypedDict(TypedDict):
    uuid: NotRequired[str]


class PutAddPeopleToDepartmentEmployees(BaseModel):
    uuid: Optional[str] = None


class PutAddPeopleToDepartmentContractorsTypedDict(TypedDict):
    uuid: NotRequired[str]


class PutAddPeopleToDepartmentContractors(BaseModel):
    uuid: Optional[str] = None


class PutAddPeopleToDepartmentRequestBodyTypedDict(TypedDict):
    version: NotRequired[str]
    r"""The current version of the object. See the [versioning guide](https://docs.gusto.com/embedded-payroll/docs/versioning#object-layer) for information on how to use this field."""
    employees: NotRequired[List[PutAddPeopleToDepartmentEmployeesTypedDict]]
    r"""Array of employees to add to the department"""
    contractors: NotRequired[List[PutAddPeopleToDepartmentContractorsTypedDict]]
    r"""Array of contractors to add to the department"""


class PutAddPeopleToDepartmentRequestBody(BaseModel):
    version: Optional[str] = None
    r"""The current version of the object. See the [versioning guide](https://docs.gusto.com/embedded-payroll/docs/versioning#object-layer) for information on how to use this field."""

    employees: Optional[List[PutAddPeopleToDepartmentEmployees]] = None
    r"""Array of employees to add to the department"""

    contractors: Optional[List[PutAddPeopleToDepartmentContractors]] = None
    r"""Array of contractors to add to the department"""


class PutAddPeopleToDepartmentRequestTypedDict(TypedDict):
    department_uuid: str
    r"""The UUID of the department"""
    request_body: PutAddPeopleToDepartmentRequestBodyTypedDict
    x_gusto_api_version: NotRequired[VersionHeader]
    r"""Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used."""


class PutAddPeopleToDepartmentRequest(BaseModel):
    department_uuid: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""The UUID of the department"""

    request_body: Annotated[
        PutAddPeopleToDepartmentRequestBody,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]

    x_gusto_api_version: Annotated[
        Optional[VersionHeader],
        pydantic.Field(alias="X-Gusto-API-Version"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = VersionHeader.TWO_THOUSAND_AND_TWENTY_FOUR_MINUS_04_MINUS_01
    r"""Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .versionheader import VersionHeader
from enum import Enum
from gusto_app_integration.types import BaseModel, Nullable, UNSET_SENTINEL
from gusto_app_integration.utils import FieldMetadata, HeaderMetadata
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class GetV1TokenInfoRequestTypedDict(TypedDict):
    x_gusto_api_version: NotRequired[VersionHeader]
    r"""Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used."""


class GetV1TokenInfoRequest(BaseModel):
    x_gusto_api_version: Annotated[
        Optional[VersionHeader],
        pydantic.Field(alias="X-Gusto-API-Version"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = VersionHeader.TWO_THOUSAND_AND_TWENTY_FOUR_MINUS_04_MINUS_01
    r"""Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used."""


class ResourceTypedDict(TypedDict):
    r"""Information about the token resource."""

    type: str
    r"""Type of object"""
    uuid: str
    r"""UUID of object"""


class Resource(BaseModel):
    r"""Information about the token resource."""

    type: str
    r"""Type of object"""

    uuid: str
    r"""UUID of object"""


class GetV1TokenInfoType(str, Enum):
    COMPANY_ADMIN = "CompanyAdmin"
    EMPLOYEE = "Employee"
    CONTRACTOR = "Contractor"


class ResourceOwnerTypedDict(TypedDict):
    r"""Information about the token owner"""

    type: GetV1TokenInfoType
    uuid: str
    r"""UUID of resource owner"""


class ResourceOwner(BaseModel):
    r"""Information about the token owner"""

    type: GetV1TokenInfoType

    uuid: str
    r"""UUID of resource owner"""


class GetV1TokenInfoResponseBodyTypedDict(TypedDict):
    r"""Example response"""

    scope: str
    r"""Space delimited string of accessible scopes."""
    resource: Nullable[ResourceTypedDict]
    r"""Information about the token resource."""
    resource_owner: Nullable[ResourceOwnerTypedDict]
    r"""Information about the token owner"""


class GetV1TokenInfoResponseBody(BaseModel):
    r"""Example response"""

    scope: str
    r"""Space delimited string of accessible scopes."""

    resource: Nullable[Resource]
    r"""Information about the token resource."""

    resource_owner: Nullable[ResourceOwner]
    r"""Information about the token owner"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = ["resource", "resource_owner"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .versionheader import VersionHeader
from enum import Enum
from gusto_app_integration.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from gusto_app_integration.utils import (
    FieldMetadata,
    HeaderMetadata,
    PathParamMetadata,
    RequestMetadata,
)
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class PutV1ContractorsContractorUUIDType(str, Enum):
    r"""The contractor type."""

    INDIVIDUAL = "Individual"
    BUSINESS = "Business"


class PutV1ContractorsContractorUUIDWageType(str, Enum):
    r"""The contractor’s wage type."""

    FIXED = "Fixed"
    HOURLY = "Hourly"


class PutV1ContractorsContractorUUIDRequestBodyTypedDict(TypedDict):
    version: str
    r"""The current version of the object. See the [versioning guide](https://docs.gusto.com/embedded-payroll/docs/idempotency) for information on how to use this field."""
    type: NotRequired[PutV1ContractorsContractorUUIDType]
    r"""The contractor type."""
    wage_type: NotRequired[PutV1ContractorsContractorUUIDWageType]
    r"""The contractor’s wage type.

    """
    start_date: NotRequired[str]
    r"""The day when the contractor will start working for the company.

    """
    hourly_rate: NotRequired[str]
    r"""The contractor’s hourly rate. This attribute is required if the wage_type is `Hourly`."""
    self_onboarding: NotRequired[bool]
    r"""Whether the contractor or the payroll admin will complete onboarding in Gusto.
    Self-onboarding is recommended so that contractors receive Gusto accounts.
    If self_onboarding is true, then email is required.
    """
    email: NotRequired[str]
    r"""The contractor’s email address."""
    first_name: NotRequired[str]
    r"""The contractor’s first name.
    This attribute is required for `Individual` contractors and will be ignored for `Business` contractors.
    """
    last_name: NotRequired[str]
    r"""The contractor’s last name.
    This attribute is required for `Individual` contractors and will be ignored for `Business` contractors.
    """
    middle_initial: NotRequired[str]
    r"""The contractor’s middle initial.
    This attribute is optional for `Individual` contractors and will be ignored for `Business` contractors.
    """
    file_new_hire_report: NotRequired[bool]
    r"""The boolean flag indicating whether Gusto will file a new hire report for the contractor.
    This attribute is optional for `Individual` contractors and will be ignored for `Business` contractors.
    """
    work_state: NotRequired[Nullable[str]]
    r"""State where the contractor will be conducting the majority of their work for the company.
    This value is used when generating the new hire report.
    This attribute is required for `Individual` contractors if `file_new_hire_report` is true and will be ignored for `Business` contractors.
    """
    ssn: NotRequired[str]
    r"""This attribute is optional for `Individual` contractors and will be ignored for `Business` contractors.
    Social security number is needed to file the annual 1099 tax form.
    """
    business_name: NotRequired[str]
    r"""The name of the contractor business. This attribute is required for `Business` contractors and will be ignored for `Individual` contractors."""
    ein: NotRequired[str]
    r"""The employer identification number of the contractor business.
    This attribute is optional for `Business` contractors and will be ignored for `Individual` contractors.
    """
    is_active: NotRequired[bool]
    r"""The status of the contractor. If the contractor's start date is in the future, updating this field to true means we are setting the start date to today."""


class PutV1ContractorsContractorUUIDRequestBody(BaseModel):
    version: str
    r"""The current version of the object. See the [versioning guide](https://docs.gusto.com/embedded-payroll/docs/idempotency) for information on how to use this field."""

    type: Optional[PutV1ContractorsContractorUUIDType] = (
        PutV1ContractorsContractorUUIDType.INDIVIDUAL
    )
    r"""The contractor type."""

    wage_type: Optional[PutV1ContractorsContractorUUIDWageType] = None
    r"""The contractor’s wage type.

    """

    start_date: Optional[str] = None
    r"""The day when the contractor will start working for the company.

    """

    hourly_rate: Optional[str] = None
    r"""The contractor’s hourly rate. This attribute is required if the wage_type is `Hourly`."""

    self_onboarding: Optional[bool] = False
    r"""Whether the contractor or the payroll admin will complete onboarding in Gusto.
    Self-onboarding is recommended so that contractors receive Gusto accounts.
    If self_onboarding is true, then email is required.
    """

    email: Optional[str] = None
    r"""The contractor’s email address."""

    first_name: Optional[str] = None
    r"""The contractor’s first name.
    This attribute is required for `Individual` contractors and will be ignored for `Business` contractors.
    """

    last_name: Optional[str] = None
    r"""The contractor’s last name.
    This attribute is required for `Individual` contractors and will be ignored for `Business` contractors.
    """

    middle_initial: Optional[str] = None
    r"""The contractor’s middle initial.
    This attribute is optional for `Individual` contractors and will be ignored for `Business` contractors.
    """

    file_new_hire_report: Optional[bool] = False
    r"""The boolean flag indicating whether Gusto will file a new hire report for the contractor.
    This attribute is optional for `Individual` contractors and will be ignored for `Business` contractors.
    """

    work_state: OptionalNullable[str] = UNSET
    r"""State where the contractor will be conducting the majority of their work for the company.
    This value is used when generating the new hire report.
    This attribute is required for `Individual` contractors if `file_new_hire_report` is true and will be ignored for `Business` contractors.
    """

    ssn: Optional[str] = None
    r"""This attribute is optional for `Individual` contractors and will be ignored for `Business` contractors.
    Social security number is needed to file the annual 1099 tax form.
    """

    business_name: Optional[str] = None
    r"""The name of the contractor business. This attribute is required for `Business` contractors and will be ignored for `Individual` contractors."""

    ein: Optional[str] = None
    r"""The employer identification number of the contractor business.
    This attribute is optional for `Business` contractors and will be ignored for `Individual` contractors.
    """

    is_active: Optional[bool] = None
    r"""The status of the contractor. If the contractor's start date is in the future, updating this field to true means we are setting the start date to today."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "type",
            "wage_type",
            "start_date",
            "hourly_rate",
            "self_onboarding",
            "email",
            "first_name",
            "last_name",
            "middle_initial",
            "file_new_hire_report",
            "work_state",
            "ssn",
            "business_name",
            "ein",
            "is_active",
        ]
        nullable_fields = ["work_state"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class PutV1ContractorsContractorUUIDRequestTypedDict(TypedDict):
    contractor_uuid: str
    r"""The UUID of the contractor"""
    request_body: PutV1ContractorsContractorUUIDRequestBodyTypedDict
    x_gusto_api_version: NotRequired[VersionHeader]
    r"""Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used."""


class PutV1ContractorsContractorUUIDRequest(BaseModel):
    contractor_uuid: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""The UUID of the contractor"""

    request_body: Annotated[
        PutV1ContractorsContractorUUIDRequestBody,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]

    x_gusto_api_version: Annotated[
        Optional[VersionHeader],
        pydantic.Field(alias="X-Gusto-API-Version"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = VersionHeader.TWO_THOUSAND_AND_TWENTY_FOUR_MINUS_04_MINUS_01
    r"""Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gusto_app_integration.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class PayrollPayrollStatusMetaTypeTypedDict(TypedDict):
    r"""Information about the payroll's status and expected dates"""

    cancellable: NotRequired[bool]
    r"""true if the payroll may be cancelled."""
    expected_check_date: NotRequired[str]
    r"""The date an employee will be paid if the payroll is submitted now."""
    initial_check_date: NotRequired[str]
    r"""The normal check date for the associated pay period."""
    expected_debit_time: NotRequired[str]
    r"""The time the employer's account will be debited if the payroll is submitted now."""
    payroll_late: NotRequired[bool]
    r"""expected_check_date > initial_check_date."""
    initial_debit_cutoff_time: NotRequired[str]
    r"""Payroll must be submitted at or before this time to avoid late payroll."""


class PayrollPayrollStatusMetaType(BaseModel):
    r"""Information about the payroll's status and expected dates"""

    cancellable: Optional[bool] = None
    r"""true if the payroll may be cancelled."""

    expected_check_date: Optional[str] = None
    r"""The date an employee will be paid if the payroll is submitted now."""

    initial_check_date: Optional[str] = None
    r"""The normal check date for the associated pay period."""

    expected_debit_time: Optional[str] = None
    r"""The time the employer's account will be debited if the payroll is submitted now."""

    payroll_late: Optional[bool] = None
    r"""expected_check_date > initial_check_date."""

    initial_debit_cutoff_time: Optional[str] = None
    r"""Payroll must be submitted at or before this time to avoid late payroll."""
